@@ -75,9 +75,7 @@ signature_check2( PKT_signature *sig, MD_HANDLE digest,
     }
     else if( get_pubkey( pk, sig->keyid ) )
 	rc = G10ERR_NO_PUBKEY;
-    else if(!pk->is_valid &&
-	    (pk->main_keyid[0]!=pk->keyid[0] ||
-	     pk->main_keyid[1]!=pk->keyid[1]))
+    else if(!pk->is_valid && !pk->is_primary)
         rc=G10ERR_BAD_PUBKEY; /* you cannot have a good sig from an
 				 invalid subkey */
     else {
@@ -521,8 +519,11 @@ check_key_signature2( KBNODE root, KBNODE node, int *is_selfsig,
 	return rc;
 
     if( sig->sig_class == 0x20 ) { /* key revocation */
-        /* designated revoker? */
-        if(pk->keyid[0]!=sig->keyid[0] || pk->keyid[1]!=sig->keyid[1])
+        u32 keyid[2];	
+	keyid_from_pk( pk, keyid );
+
+	/* is it a designated revoker? */
+        if(keyid[0]!=sig->keyid[0] || keyid[1]!=sig->keyid[1])
 	  rc=check_revocation_keys(pk,sig);
 	else
 	  {
